You've been invited to join {{ account }} on {{ site_name }} by {{ sender.first_name }} {{ sender.last_name }}.

Follow this link to create your user account.

http://{{ domain }}{% url invitations_register user.id token %}

If you are unsure about this link please contact the sender.
